use discrete_logarithm::discrete_log_pollard_rho;
use keygen::Person;
use rug::Integer;
use utils::mod_pow;

mod keygen;
mod utils;

fn main() {
    let mut alice = Person::new("alice");
    let mut bob = Person::new("bob");
    Person::diffie_hellman(&mut alice, &mut bob);
    alice.send(b"Hello World", &bob);

    let alice_priv = discrete_log_pollard_rho(
        &Integer::from(alice.r#mod.unwrap()),
        &Integer::from(alice.public_key.unwrap()),
        &Integer::from(alice.gen.unwrap()),
        None,
    ).unwrap();
    let bob_priv = discrete_log_pollard_rho(
        &Integer::from(bob.r#mod.unwrap()),
        &Integer::from(bob.public_key.unwrap()),
        &Integer::from(bob.gen.unwrap()),
        None,
    );
    eprintln!(
        "[mallory] Calculated {}'s private key {} from {}'s public key {}",
        alice.name,
        &alice_priv,
        alice.name,
        alice.public_key.unwrap()
    );
    eprintln!(
        "[mallory] Calculated {}'s private key {} from {}'s public key {}",
        bob.name,
        bob_priv.unwrap(),
        bob.name,
        bob.public_key.unwrap()
    );
    eprintln!(
        "[mallory] Calculated {}'s and {}'s shared secret {}",
        alice.name,
        bob.name,
        mod_pow(bob.public_key.unwrap(), alice_priv.to_u32_wrapping(), alice.r#mod.unwrap())
    );
    //Some(mod_pow(public, self.private_key.unwrap(), r#mod
}
